#version 150 core

uniform sampler2D albedoFirstBuffer;
uniform sampler2D normalFirstBuffer;
uniform sampler2D lightFirstBuffer;
uniform sampler2D ssaoBuffer;
uniform sampler2D depthFirstBuffer;
uniform sampler2D albedoSecondBuffer;
//uniform sampler2D normalSecondBuffer;
uniform sampler2D lightSecondBuffer;
uniform sampler2D depthSecondBuffer;

uniform int drawMode;
uniform float gammaCorrection;
uniform bool drawSecondGeometryLayer;
uniform bool drawLights;
uniform bool drawSSAO;
uniform vec3 ambientIntensity;
//vec3 ambientIntensity = vec3(0.01);


out vec4 color;

vec3 screenGamma = vec3( 1.0f / gammaCorrection );
const float exposure = 4.0f;

void main() {
	ivec2 ssCoord = ivec2( gl_FragCoord.xy );

	if ( drawMode == 1 ) {
		vec4 albedoFirst = texelFetch( albedoFirstBuffer, ssCoord, 0 );
		vec3 colorFirst  = albedoFirst.rgb;
		if ( drawLights ) {
			vec3 lightFirst = texelFetch( lightFirstBuffer, ssCoord, 0 ).xyz;
			float ssao      = 1.0f;
			if ( drawSSAO ) {
				ssao        = texelFetch( ssaoBuffer, ssCoord, 0 ).x;
			}
			colorFirst *= lightFirst + ambientIntensity * ssao;
		}

		if ( drawSecondGeometryLayer ) {
			if ( albedoFirst.a < 1.0f ) {
				vec3 colorSecond = texelFetch( albedoSecondBuffer, ssCoord, 0 ).rgb;
				if ( drawLights ) {
					vec3 lightSecond = texelFetch( lightSecondBuffer, ssCoord, 0 ).xyz;
					colorSecond     *= lightSecond + ambientIntensity;
				}
				colorFirst = mix( colorSecond, colorFirst, albedoFirst.a );
			}
		}

		// Exposure tone mapping
		vec3 mapped = vec3( 1.0f ) - exp( -colorFirst * exposure );
		color = vec4( pow( mapped, screenGamma ), 1.0f );
	}
	else if ( drawMode == 2 ) {
		color = texelFetch( normalFirstBuffer, ssCoord, 0 );
	}
	else if ( drawMode == 3 ) {
		float ssao = 1.0f;
		if ( drawSSAO ) {
			ssao   = texelFetch( ssaoBuffer, ssCoord, 0 ).x;
		}
		vec3 light = texelFetch( lightFirstBuffer, ssCoord, 0 ).rgb + ( ambientIntensity * ssao );
		color = vec4( pow( light, screenGamma ), 1.0f );
	}
	else if ( drawMode == 4 ) {
		color = vec4( vec3( texelFetch( ssaoBuffer, ssCoord, 0 ).x ), 1.0f );
	}
	else if ( drawMode == 5 ) {
		color = vec4( vec3( texelFetch( depthFirstBuffer, ssCoord, 0 ).x ), 1.0f );
	}
	else if ( drawMode == 6 ) {
		vec3 albedo = texelFetch( albedoSecondBuffer, ssCoord, 0 ).rgb;
		// Exposure tone mapping
		vec3 mapped = vec3( 1.0f ) - exp( -albedo * exposure );
		color = vec4( pow( mapped, screenGamma ), 1.0f );
	}
	/*else if ( drawMode == 7 ) {
		color = texelFetch( normalSecondBuffer, ssCoord, 0 );
	}*/
	else if ( drawMode == 7 ) {
		vec3 light = texelFetch( lightSecondBuffer, ssCoord, 0 ).rgb + ambientIntensity;
		color = vec4( pow( light, screenGamma ), 1.0f );
	}
	else if ( drawMode == 8 ) {
		color = vec4( vec3( texelFetch( depthSecondBuffer, ssCoord, 0 ).x ), 1.0f );
	}
}
