#version 150 core
#extension GL_ARB_explicit_attrib_location: enable

in vec2 textureCoord;
in mat3 tbnMatrix;

uniform sampler2D textureAlbedo;
uniform sampler2D textureNormal;
uniform sampler2D depthFirstBuffer;

layout ( location = 0 ) out vec3 albedoBuffer;
layout ( location = 1 ) out vec3 normalBuffer;

vec3 getNormal( in vec2 texCoord ) {
	vec3 tsNormal = texture( textureNormal, texCoord ).xyz;
	// переводим в NDC (-1.0f, 1.0f)
	vec3 csNormal = normalize( tsNormal * 2.0 - 1.0 );
	// переводим в мировое пространство
	return normalize( tbnMatrix * csNormal );
}

void main() {
	float firstDepth = texelFetch( depthFirstBuffer, ivec2( gl_FragCoord.xy ), 0 ).x;
	vec4 colorLinear = texture( textureAlbedo, textureCoord );
	// удаляем полигоны первого слоя и любые полупрозрачные полигоны...
	if ( firstDepth == gl_FragCoord.z || colorLinear.a < 1.0f ) {
		discard;
	}
	albedoBuffer = colorLinear.rgb;
	normalBuffer = getNormal( textureCoord );
}
